import AsyncStorage from '@react-native-async-storage/async-storage';
const {
  openDatabase
} = require('react-native-sqlite-storage');

//SQLite.DEBUG(config.app.isDebug)

/**
 * Administra la base de datos.
 */
export class DB {
  static _instances = new Map();
  /**
   * Abre la base de datos
   * 
   * @param {string} name nombre de la db
   */
  constructor(name) {
    this.name = name;
  }
  static get(name) {
    var db = this._instances.get(name);
    if (!db) {
      db = new DB(name);
      this._instances.set(name, db);
    }
    return db;
  }

  /**
   * Abre la base de datos
   * @returns {SQLiteDatabase}
   */
  async open() {
    if (!this.sqlite) {
      //console.debug("DB.open", this.name)
      this.sqlite = await openDatabase({
        "name": this.name
      });
    }
    return this.sqlite;
  }

  /**
   * Cierra la db
   */
  async close() {
    if (this.sqlite) {
      this.sqlite.close();
      this.sqlite = null;
    }
  }

  /**
   * Ejecuta sentecias sql.
   * 
   * @param {string} sql sentencia
   * @param {any[]} params parametros
   * 
   * @returns {
   *  "insertId": $insertId, 
   *  "rows": [
   *   {"$columnName": $value}
   *  ]
   * }
   */
  executeSql(sql) {
    let params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    return new Promise((txResolve, txReject) => {
      var _this$sqlite;
      (_this$sqlite = this.sqlite) === null || _this$sqlite === void 0 || _this$sqlite.transaction(tx => {
        new Promise((sqlResolve, sqlReject) => {
          tx.executeSql(sql, params, (_tx, result) => {
            sqlResolve({
              rows: result.rows.raw(),
              insertId: result.insertId,
              rowsAffected: result.rowsAffected
            });
          }, error => {
            sqlReject(error);
          });
        }).then(txResolve).catch(txReject);
      });
    });
  }

  /**
   * Obtiene el numero de version de la db
   */
  getVersion() {
    return this.executeSql("PRAGMA user_version").then(_ref => {
      let {
        rows
      } = _ref;
      return rows[0];
    }).then(row => row.user_version ?? 0);
  }

  /**
   * Establece la versión de la base de datos.
   * @param {int} version la nueva versión de la base de datos
   */
  setVersion(version) {
    return this.executeSql("PRAGMA user_version = " + version).then(_ref2 => {
      let {
        rows
      } = _ref2;
      return rows[0];
    });
  }

  /**
   * Obtiene un instacia de Table
   */
  table(name) {
    return new Table(this, name);
  }

  /**
   * Inicializa el scheme de la base de datos.
   */
  async init(scheme, version) {
    console.debug("DB.init");
    await this.open();

    // Obtiene la version actual de la db
    const dbVersion = await this.getVersion();

    // Valida la version actual con la version del archivo
    if (dbVersion == 0) {
      await scheme.onCreate(this);
    } else if (dbVersion != version) {
      await scheme.onUpdate(this, dbVersion, version);
    }

    // Setea la nueva version en la base de datos
    this.setVersion(version);

    // Siembra los datos en la db.
    // Validamso desde el storage si aun no se a sembrado una semilla 
    const value = await AsyncStorage.getItem(`db.seeder.${this.name}`);
    if (!value) {
      // guarda un true en el storage indicando que se a sembrado la semilla
      await AsyncStorage.setItem(`db.seeder.${this.name}`, "true");
      // llama la funcion para sembrar
      await scheme.seeder(this);
    }
  }

  /**
   * Compila una clausula where
   * 
   * @param {any[]} array [1,3,4,2]
   * @returns (?, ?, ?, ?)
   */
  static whereClausePlaceholders(array) {
    var args = [];
    for (let i = 0; i < array.length; i++) {
      args.push("?");
    }
    return "(" + args.join(", ") + ")";
  }

  /**
   * @param sqlString "valor con 'comillas'"
   * @returns "'valor con \'comillas\''"
   */
  static escapeSqlString(sqlString) {
    let escape = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '\'';
    var sb = [];
    sb.push(escape);
    if (sqlString.indexOf(escape) != -1) {
      var length = sqlString.length;
      for (var i = 0; i < length; i++) {
        var c = sqlString.charAt(i);
        if (c == escape) {
          sb.push(escape);
        }
        sb.push(c);
      }
    } else {
      sb.push(sqlString);
    }
    sb.push(escape);
    return sb.join("");
  }
}

/**
 * Gestiona las propiedades de un tabla de Sqlite
 */
export class Table {
  constructor(db, name) {
    this.db = db;
    this.name = name;
  }

  /**
   * ```
   *  cid 	name      type 	  notnull 	dflt_value 	  pk
   *    0   id      integer 	      0 	      null 	    1
   *    1   type       text 	      0 	      null 	    0
   *    2   data       json 	      0 	      null 	    0
   * ```
   * 
   * @param name nombre de la tabla
   * @returns 
   */
  info() {
    return this.db.executeSql(`PRAGMA table_info(${this.name})`).then(_ref3 => {
      let {
        rows
      } = _ref3;
      return rows;
    });
  }

  /**
   * Agrega las columnas que no existen
   * @param db 
   * @param columns 
   * @returns 
   */
  async addColumns(columns) {
    const currentColumns = await this.info();
    return await columns.filter(async columnOptions => {
      for (let i = 0; i < currentColumns.length; i++) {
        const currentColumn = currentColumns[i];
        if (columnOptions.name == currentColumn.name) {
          return false; // ya existe la columna en la tabla
        }
      }

      const sqlColumn = columnOptions.compile();
      const sql = `ALTER TABLE ${this.name} ADD COLUMN ${sqlColumn};`;
      console.debug(sql);
      await this.db.executeSql(sql);
      return true; // no existia la columna en la tabla
    });
  }
}

/**
* id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAUL '' +
*/
export class Column {
  _primaryKey = false;
  _autoIncrement = false;
  _nullable = false;
  constructor(name, type) {
    this.name = name;
    this.type = type;
  }
  primaryKey() {
    this._primaryKey = true;
    return this;
  }
  autoIncrement() {
    this._autoIncrement = true;
    return this;
  }
  defaultVal(defaultVal) {
    this._defaultVal = defaultVal;
    return this;
  }
  nullable() {
    this._nullable = true;
    return this;
  }
  compile() {
    var sql = [];
    // column-def:
    sql.push(this.name);
    sql.push(" ");
    sql.push(this.type);
    // column-constraint:
    if (this._primaryKey) sql.push(" PRIMARY KEY");
    if (this._autoIncrement) sql.push(" AUTOINCREMENT");
    sql.push(this._nullable ? " NULL" : " NOT NULL");
    if (this._defaultVal != null) {
      sql.push(" DEFAULT ");
      sql.push(DB.escapeSqlString(this._defaultVal));
    }
    return sql.join("");
  }
}

/**
 * Esquema para la base de datos
 */

/**
 * Resultado de los querys
 */
//# sourceMappingURL=index.js.map