export type SQLiteDatabase = {
    close(): () => void;
    transaction: (tx: any) => void;
};
/**
 * Administra la base de datos.
 */
export declare class DB {
    private static _instances;
    name: string;
    sqlite?: SQLiteDatabase | null;
    /**
     * Abre la base de datos
     *
     * @param {string} name nombre de la db
     */
    constructor(name: string);
    static get(name: string): DB;
    /**
     * Abre la base de datos
     * @returns {SQLiteDatabase}
     */
    open(): Promise<SQLiteDatabase>;
    /**
     * Cierra la db
     */
    close(): Promise<void>;
    /**
     * Ejecuta sentecias sql.
     *
     * @param {string} sql sentencia
     * @param {any[]} params parametros
     *
     * @returns {
     *  "insertId": $insertId,
     *  "rows": [
     *   {"$columnName": $value}
     *  ]
     * }
     */
    executeSql(sql: string, params?: any[]): Promise<ResultSet>;
    /**
     * Obtiene el numero de version de la db
     */
    getVersion(): Promise<number>;
    /**
     * Establece la versión de la base de datos.
     * @param {int} version la nueva versión de la base de datos
     */
    setVersion(version: number): Promise<any>;
    /**
     * Obtiene un instacia de Table
     */
    table(name: string): Table;
    /**
     * Inicializa el scheme de la base de datos.
     */
    init(scheme: ItScheme, version: number): Promise<void>;
    /**
     * Compila una clausula where
     *
     * @param {any[]} array [1,3,4,2]
     * @returns (?, ?, ?, ?)
     */
    static whereClausePlaceholders(array: any[]): string;
    /**
     * @param sqlString "valor con 'comillas'"
     * @returns "'valor con \'comillas\''"
     */
    static escapeSqlString(sqlString: string, escape?: string): string;
}
/**
 * Gestiona las propiedades de un tabla de Sqlite
 */
export declare class Table {
    db: DB;
    name: string;
    constructor(db: DB, name: string);
    /**
     * ```
     *  cid 	name      type 	  notnull 	dflt_value 	  pk
     *    0   id      integer 	      0 	      null 	    1
     *    1   type       text 	      0 	      null 	    0
     *    2   data       json 	      0 	      null 	    0
     * ```
     *
     * @param name nombre de la tabla
     * @returns
     */
    info(): Promise<any[]>;
    /**
     * Agrega las columnas que no existen
     * @param db
     * @param columns
     * @returns
     */
    addColumns(columns: Column[]): Promise<Column[]>;
}
/**
* id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAUL '' +
*/
export declare class Column {
    name: string;
    type: string;
    private _primaryKey;
    private _autoIncrement;
    private _nullable;
    private _defaultVal?;
    constructor(name: string, type: string);
    primaryKey(): Column;
    autoIncrement(): Column;
    defaultVal(defaultVal: string): Column;
    nullable(): Column;
    compile(): string;
}
/**
 * Esquema para la base de datos
 */
export interface ItScheme {
    onCreate(db: DB): void;
    onUpdate(db: DB, oldVersion: number, newVersion: number): void;
    seeder(db: DB): void;
}
/**
 * Resultado de los querys
 */
export type ResultSet = {
    insertId?: number;
    rowsAffected: number;
    rows: any[];
};
//# sourceMappingURL=index.d.ts.map