import AsyncStorage from '@react-native-async-storage/async-storage';
const { openDatabase } = require('react-native-sqlite-storage')

//SQLite.DEBUG(config.app.isDebug)

export type SQLiteDatabase = {
  close(): () => void
  transaction: (tx: any) => void
}

/**
 * Administra la base de datos.
 */
export class DB {
  private static _instances = new Map<string, DB>();

  public name: string
  public sqlite?: SQLiteDatabase | null

  /**
   * Abre la base de datos
   * 
   * @param {string} name nombre de la db
   */
  constructor(name: string) {
    this.name = name;
  }

  public static get(name: string): DB {
    var db = this._instances.get(name);
    if (!db) {
      db = new DB(name);
      this._instances.set(name, db);
    }
    return db;
  }

  /**
   * Abre la base de datos
   * @returns {SQLiteDatabase}
   */
  async open(): Promise<SQLiteDatabase> {
    if (!this.sqlite) {
      //console.debug("DB.open", this.name)
      this.sqlite = await openDatabase({ "name": this.name });
    }
    return this.sqlite!;
  }

  /**
   * Cierra la db
   */
  async close() {
    if (this.sqlite) {
      this.sqlite.close();
      this.sqlite = null;
    }
  }
 
  /**
   * Ejecuta sentecias sql.
   * 
   * @param {string} sql sentencia
   * @param {any[]} params parametros
   * 
   * @returns {
   *  "insertId": $insertId, 
   *  "rows": [
   *   {"$columnName": $value}
   *  ]
   * }
   */
  executeSql(
    sql: string, 
    params: any[] = []
  ): Promise<ResultSet> 
  {
    return new Promise((txResolve, txReject) => {
      this.sqlite?.transaction((tx: any /*Transaction*/) => {
        new Promise<ResultSet>((sqlResolve, sqlReject) => {
          tx.executeSql(sql, params,
            (_tx: any, result: any) => {
              sqlResolve({ 
                rows: result.rows.raw(), 
                insertId: result.insertId,
                rowsAffected: result.rowsAffected
              })
            },
            (error: any) => { sqlReject(error) }
          )
        }).then(txResolve).catch(txReject)
      });
    });
  }

  /**
   * Obtiene el numero de version de la db
   */
  getVersion(): Promise<number> {
    return this.executeSql("PRAGMA user_version")
      .then(({rows}) => rows[0])
      .then((row) => row.user_version ?? 0);
  }

  /**
   * Establece la versión de la base de datos.
   * @param {int} version la nueva versión de la base de datos
   */
  setVersion(version: number) {
    return this.executeSql("PRAGMA user_version = " + version)
      .then(({rows}) => rows[0]);
  }

  /**
   * Obtiene un instacia de Table
   */
  table(name: string): Table {
    return new Table(this, name)
  }

  /**
   * Inicializa el scheme de la base de datos.
   */
  async init(scheme: ItScheme, version: number) {
    console.debug("DB.init");
    await this.open()

    // Obtiene la version actual de la db
    const dbVersion: number = await this.getVersion();

    // Valida la version actual con la version del archivo
    if (dbVersion == 0) {
      await scheme.onCreate(this);

    } else if (dbVersion != version) {
      await scheme.onUpdate(this, dbVersion, version);
    }

    // Setea la nueva version en la base de datos
    this.setVersion(version);
    
    // Siembra los datos en la db.
    // Validamso desde el storage si aun no se a sembrado una semilla 
    const value = await AsyncStorage.getItem(`db.seeder.${this.name}`);
    if (!value) {
      // guarda un true en el storage indicando que se a sembrado la semilla
      await AsyncStorage.setItem(`db.seeder.${this.name}`, "true"); 
      // llama la funcion para sembrar
      await scheme.seeder(this); 
    }
  }

  /**
   * Compila una clausula where
   * 
   * @param {any[]} array [1,3,4,2]
   * @returns (?, ?, ?, ?)
   */
  static whereClausePlaceholders(array: any[]): string {
    var args: string[] = [];
    for (let i = 0; i < array.length; i++) {
      args.push("?");
    }
    return "(" + args.join(", ") + ")";
  }

  /**
   * @param sqlString "valor con 'comillas'"
   * @returns "'valor con \'comillas\''"
   */
  static escapeSqlString(sqlString: string, escape = '\''): string {
    var sb: any[] = []
    sb.push(escape);

    if (sqlString.indexOf(escape) != -1) {
      var length: number = sqlString.length;
      for (var i = 0; i < length; i++) {
          var c: string = sqlString.charAt(i);
          if (c == escape) {
            sb.push(escape);
          }
          sb.push(c);
      }
    } else {
      sb.push(sqlString);
    }

    sb.push(escape);
    return sb.join("")
  }
}

/**
 * Gestiona las propiedades de un tabla de Sqlite
 */
export class Table {

  public db: DB;
  public name: string

  constructor(db: DB, name: string) {
    this.db = db;
    this.name = name
  }

  /**
   * ```
   *  cid 	name      type 	  notnull 	dflt_value 	  pk
   *    0   id      integer 	      0 	      null 	    1
   *    1   type       text 	      0 	      null 	    0
   *    2   data       json 	      0 	      null 	    0
   * ```
   * 
   * @param name nombre de la tabla
   * @returns 
   */
  info(): Promise<any[]> {
    return this.db.executeSql(`PRAGMA table_info(${this.name})`)
    .then(({rows}) => rows);
  }

  /**
   * Agrega las columnas que no existen
   * @param db 
   * @param columns 
   * @returns 
   */
  async addColumns(columns: Column[]): Promise<Column[]> {
    const currentColumns = await this.info()
    
    return await columns.filter(async (columnOptions: Column) => {
      for (let i = 0; i < currentColumns.length; i++) {
        const currentColumn = currentColumns[i];
        if (columnOptions.name == currentColumn.name) {
          return false; // ya existe la columna en la tabla
        }
      }

      const sqlColumn = columnOptions.compile()
      const sql = `ALTER TABLE ${this.name} ADD COLUMN ${sqlColumn};`
      console.debug(sql)
      await this.db.executeSql(sql)

      return true; // no existia la columna en la tabla
    });
  }
}


/**
* id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL DEFAUL '' +
*/
export class Column {
  public name: string
  public type: string
  private _primaryKey: boolean = false
  private _autoIncrement: boolean = false
  private _nullable: boolean = false
  private _defaultVal?: string

  constructor(name: string, type: string) {
    this.name = name
    this.type = type;
  }

  primaryKey(): Column {
    this._primaryKey = true
    return this
  }

  autoIncrement(): Column {
    this._autoIncrement = true
    return this
  }

  defaultVal(defaultVal: string): Column {
    this._defaultVal = defaultVal
    return this
  }

  nullable(): Column {
    this._nullable = true
    return this
  }

  compile(): string {
    var sql: any[] = []
    // column-def:
    sql.push(this.name)
    sql.push(" ")
    sql.push(this.type)
    // column-constraint:
    if (this._primaryKey) sql.push(" PRIMARY KEY")
    if (this._autoIncrement) sql.push(" AUTOINCREMENT")

    sql.push((this._nullable) ? " NULL" : " NOT NULL")
    if (this._defaultVal != null) {
      sql.push(" DEFAULT ")
      sql.push(DB.escapeSqlString(this._defaultVal))
    }
    return sql.join("")
  }
}

/**
 * Esquema para la base de datos
 */
export interface ItScheme {
  onCreate(db: DB): void;
  onUpdate(db: DB, oldVersion: number, newVersion: number): void;
  seeder(db: DB): void;
}

/**
 * Resultado de los querys
 */
export type ResultSet = {
  insertId?: number;
  rowsAffected: number;
  rows: any[];
}